@@ -1,4 +1,4 @@
-CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
+CBPS.3Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
 {
   probs.min<-1e-6
   
@@ -13,11 +13,10 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     ##Designate a few objects in the function.
     beta.curr<-matrix(beta.curr,k,no.treats-1)
     X<-as.matrix(X.gmm)
-    
+  
     ##Designate sample size, number of treated and control observations,
     ##theta.curr, which are used to generate probabilities.
     ##Trim probabilities, and generate weights.
-    n<-c(sum(T1), sum(T2), sum(T3))
     theta.curr<-X%*%beta.curr
     baseline.prob<-apply(theta.curr,1,function(x) (1+sum(exp(x)))^-1)
     probs.curr<-cbind(baseline.prob, exp(theta.curr[,1])*baseline.prob, exp(theta.curr[,2])*baseline.prob)
@@ -32,13 +31,13 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/sum(n)*t(X)%*%w.curr
+    w.curr.del<-1/n*t(X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/sum(n)*t(X)%*%(T2-probs.curr[,2]), 
-            1/sum(n)*t(X)%*%(T3-probs.curr[,3]),
+    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
+            1/n*t(X)%*%(T3-probs.curr[,3]),
             w.curr.del)
     
     
@@ -57,17 +56,17 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
       X.3.4<-X*(-probs.curr[,2]^-1+probs.curr[,3]^-1)
       X.4.4<-X*(probs.curr[,2]^-1+probs.curr[,3]^-1)
       
-      V<-1/sum(n)*rbind(cbind(t(X.1.1)%*%X,t(X.1.2)%*%X,t(X.1.3)%*%X,t(X.1.4)%*%X),
-                        cbind(t(X.1.2)%*%X,t(X.2.2)%*%X,t(X.2.3)%*%X,t(X.2.4)%*%X),
-                        cbind(t(X.1.3)%*%X,t(X.2.3)%*%X,t(X.3.3)%*%X,t(X.3.4)%*%X),
-                        cbind(t(X.1.4)%*%X,t(X.2.4)%*%X,t(X.3.4)%*%X,t(X.4.4)%*%X))
+      V<-1/n*rbind(cbind(t(X.1.1)%*%X,t(X.1.2)%*%X,t(X.1.3)%*%X,t(X.1.4)%*%X),
+                   cbind(t(X.1.2)%*%X,t(X.2.2)%*%X,t(X.2.3)%*%X,t(X.2.4)%*%X),
+                   cbind(t(X.1.3)%*%X,t(X.2.3)%*%X,t(X.3.3)%*%X,t(X.3.4)%*%X),
+                   cbind(t(X.1.4)%*%X,t(X.2.4)%*%X,t(X.3.4)%*%X,t(X.4.4)%*%X))
       
       invV<-ginv(V)
     }
     
     ##Calculate the GMM loss.
     loss1<-as.vector(t(gbar)%*%invV%*%(gbar))      
-    out1<-list("loss"=max(loss1*sum(n),loss1*sum(n)), "invV"=invV)
+    out1<-list("loss"=loss1, "invV"=invV)
     out1
   }
   gmm.loss<-function(x,...) gmm.func(x,...)$loss
@@ -85,9 +84,10 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     probs.curr<-probs.curr/norms
     
     w.curr<-cbind(2*T1/probs.curr[,1] - T2/probs.curr[,2] - T3/probs.curr[,3],
-                  T2/probs.curr[,2] - T3/probs.curr[,3])
+                  T2/probs.curr[,2] - T3/probs.curr[,3])/n
+    
     ##Generate mean imbalance.
-    loss1<-sum(diag(abs(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))))
+    loss1<-sum(diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr)))
     loss1
   }
   
@@ -100,7 +100,6 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     ##Designate sample size, number of treated and control observations,
     ##theta.curr, which are used to generate probabilities.
     ##Trim probabilities, and generate weights.
-    n<-c(sum(T1), sum(T2), sum(T3))
     theta.curr<-X%*%beta.curr
     baseline.prob<-apply(theta.curr,1,function(x) (1+sum(exp(x)))^-1)
     probs.curr<-cbind(baseline.prob, exp(theta.curr[,1])*baseline.prob, exp(theta.curr[,2])*baseline.prob)
@@ -115,25 +114,25 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/sum(n)*t(X)%*%w.curr
+    w.curr.del<-1/n*t(X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/sum(n)*t(X)%*%(T2-probs.curr[,2]), 
-            1/sum(n)*t(X)%*%(T3-probs.curr[,3]),
+    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
+            1/n*t(X)%*%(T3-probs.curr[,3]),
             w.curr.del)
     
-    dgbar<-rbind(cbind(1/sum(n)*t(apply(t(X),1,function(x) -x*probs.curr[,2]*(1-probs.curr[,2])))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,3]))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) x*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) x*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])))%*%X),
-                 cbind(1/sum(n)*t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,3]))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) -x*probs.curr[,3]*(1-probs.curr[,3])))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) x*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])))%*%X,
-                       1/sum(n)*t(apply(t(X),1,function(x) x*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])))%*%X))
+    dgbar<-rbind(cbind(1/n*t(-X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
+                       1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X,
+                       1/n*t(X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))%*%X),
+                 cbind(1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(-X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
+                       1/n*t(X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X,
+                       1/n*t(X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))%*%X))
                  
-    out<-2*sum(n)*dgbar%*%invV%*%gbar
+    out<-2*dgbar%*%invV%*%gbar
     out
   }
   
@@ -150,37 +149,31 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     probs.curr<-probs.curr/norms
     
     w.curr<-cbind(2*T1/probs.curr[,1] - T2/probs.curr[,2] - T3/probs.curr[,3],
-                  T2/probs.curr[,2] - T3/probs.curr[,3])
+                  T2/probs.curr[,2] - T3/probs.curr[,3])/n
     
-    dw.beta1<-cbind(t(apply(t(X),1,function(x) x*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))),
-                    t(apply(t(X),1,function(x) x*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3]))))
-    dw.beta2<-cbind(t(apply(t(X),1,function(x) x*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))),
-                    t(apply(t(X),1,function(x) x*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3]))))
+    dw.beta1<-cbind(t(X*(2*T1*probs.curr[,2]/probs.curr[,1] + T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])),
+                    t(X*(-T2*(1-probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3])))/n
+    dw.beta2<-cbind(t(X*(2*T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])),
+                    t(X*(T2*probs.curr[,3]/probs.curr[,2] + T3*(1-probs.curr[,3])/probs.curr[,3])))/n
     ##Generate mean imbalance.
     loss1<-diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))
-    out.1<-sapply(2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]), 
-                  function(x) ifelse((x > 0 & loss1[1] > 0) | (x < 0 & loss1[1] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]),
-             function(x) ifelse((x > 0 & loss1[2] > 0) | (x < 0 & loss1[2] < 0),abs(x),-abs(x)))
-    out.2<-sapply(2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]), 
-                  function(x) ifelse((x > 0 & loss1[1] > 0) | (x < 0 & loss1[1] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]),
-             function(x) ifelse((x > 0 & loss1[2] > 0) | (x < 0 & loss1[2] < 0),abs(x),-abs(x)))
+    out.1<-2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) + 
+      2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2])
+    out.2<-2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
+      2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2])
     out<-c(out.1, out.2)
     out
   }
   
   n<-length(treat)
-  n.t<-c(sum(T1),sum(T2),sum(T3))
-  x.orig<-x<-cbind(as.matrix(X))
-  
+
   ##Run multionmial logit
   dat.dummy<-data.frame(treat=treat,X)
   #Need to generalize for different dimensioned X's
-  xnam<- paste0("X", 1:dim(X)[2])
+  xnam<- colnames(dat.dummy[,-1])
   fmla <- as.formula(paste("as.factor(treat) ~ -1 + ", paste(xnam, collapse= "+")))
   mnl1<-multinom(fmla, data=dat.dummy, trace=FALSE)
-  mcoef<-matrix(coef(mnl1),k,no.treats-1,byrow=TRUE)
+  mcoef<-t(coef(mnl1))
   mcoef[is.na(mcoef[,1]),1]<-0
   mcoef[is.na(mcoef[,2]),2]<-0
   probs.mnl<-cbind(1/(1+exp(X%*%mcoef[,1])+exp(X%*%mcoef[,2])),
@@ -194,20 +187,15 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   probs.mnl<-probs.mnl/norms
   
   mnl1$fit<-matrix(probs.mnl,nrow=n,ncol=no.treats)
-  beta.curr<-mcoef
+  beta.curr<-matrix(mcoef, ncol = 1)
   beta.curr[is.na(beta.curr)]<-0
-  
+
   alpha.func<-function(alpha) gmm.loss(beta.curr*alpha)
   beta.curr<-beta.curr*optimize(alpha.func,interval=c(.8,1.1))$min
   
-  if(twostep)
-  {
-    this.invV<-gmm.func(beta.curr)$invV
-  }
-  
-  
   ##Generate estimates for balance and CBPSE
   gmm.init<-beta.curr
+  this.invV<-gmm.func(gmm.init)$invV
   
   if (twostep)
   {
@@ -265,7 +253,6 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   norms<-apply(probs.opt,1,sum)
   probs.opt<-probs.opt/norms
   
-  
   J.opt<-ifelse(twostep, gmm.func(beta.opt, invV = this.invV)$loss, gmm.func(beta.opt)$loss)
   
   if ((J.opt > gmm.loss(mcoef)) & (bal.loss(beta.opt) > bal.loss(mcoef)))
@@ -276,14 +263,12 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     warning("Optimization failed.  Results returned are for MLE.")
   }
   
-  ##Generate weights
-  w.opt<-rbind(2*T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3],
-               T2/probs.opt[,2] - T3/probs.opt[,3])
-  
+
   residuals<-cbind(T1-probs.opt[,1],T2-probs.opt[,2],T3-probs.opt[,3])
   deviance <- -2*c(sum(T1*log(probs.opt[,1])+T2*log(probs.opt[,2])+T3*log(probs.opt[,3])))
   nulldeviance <- -2*c(sum(T1*log(mean(T1))+T2*log(mean(T2))+T3*log(mean(T3))))
   
+  ##Generate weights
   norm1<-norm2<-norm3<-1
   if (standardize)
   {
@@ -291,20 +276,14 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     norm2<-sum(T2/probs.opt[,2])
     norm3<-sum(T3/probs.opt[,3])
   }
-  w<-T1/probs.opt[,1]/norm1 + T2/probs.opt[,2]/norm2 + T3/probs.opt[,3]/norm3
+  w.opt<-T1/probs.opt[,1]/norm1 + T2/probs.opt[,2]/norm2 + T3/probs.opt[,3]/norm3
   
-  if (twostep)
-  {
-    W<-this.invV
-  }
-  else
-  {
-    W<-gmm.func(beta.opt)$invV
-  }
+  W<-gmm.func(beta.opt)$invV
+
   XG.1.1<-t(-X*probs.opt[,2]*(1-probs.opt[,2]))%*%X
   XG.1.2<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
   XG.1.3<-t(X*(2*T1*probs.opt[,2]/probs.opt[,1] + T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
-  XG.1.4<-t(X*(T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
+  XG.1.4<-t(X*(-T2*(1-probs.opt[,2])/probs.opt[,2] - T3*probs.opt[,2]/probs.opt[,3]))%*%X
   XG.2.1<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
   XG.2.2<-t(-X*probs.opt[,3]*(1-probs.opt[,3]))%*%X
   XG.2.3<-t(X*(2*T1*probs.opt[,3]/probs.opt[,1] - T2*probs.opt[,3]/probs.opt[,2] + T3*(1-probs.opt[,3])/probs.opt[,3]))%*%X
@@ -323,7 +302,7 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   
   class(beta.opt) <- "coef"
   
-  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w, 
+  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w.opt, 
                "y"=treat,"x"=X,"converged"=opt1$conv,"J"=J.opt,"var"=vcov, 
                "mle.J"=ifelse(twostep, gmm.func(mcoef, invV = this.invV)$loss, gmm.loss(mcoef)))
   
@@ -332,7 +311,7 @@ CBPS.3Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   output
 }
 
-CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
+CBPS.4Treat<-function(treat, X, method, k, XprimeX.inv, bal.only, iterations, standardize, twostep, ...)
 {
   probs.min<-1e-6
   
@@ -352,7 +331,6 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     ##Designate sample size, number of treated and control observations,
     ##theta.curr, which are used to generate probabilities.
     ##Trim probabilities, and generate weights.
-    n<-c(sum(T1), sum(T2), sum(T3), sum(T4))
     theta.curr<-X%*%beta.curr
     baseline.prob<-apply(theta.curr,1,function(x) (1+sum(exp(x)))^-1)
     probs.curr<-cbind(baseline.prob, exp(theta.curr[,1])*baseline.prob, exp(theta.curr[,2])*baseline.prob, exp(theta.curr[,3])*baseline.prob)
@@ -368,14 +346,14 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
                   -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])    
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/sum(n)*t(X)%*%w.curr
+    w.curr.del<-1/n*t(X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/sum(n)*t(X)%*%(T2-probs.curr[,2]), 
-            1/sum(n)*t(X)%*%(T3-probs.curr[,3]),
-            1/sum(n)*t(X)%*%(T4-probs.curr[,4]),
+    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
+            1/n*t(X)%*%(T3-probs.curr[,3]),
+            1/n*t(X)%*%(T4-probs.curr[,4]),
             w.curr.del)
     
     
@@ -405,7 +383,7 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
       X.5.6<-X*(-probs.curr[,1]^-1-probs.curr[,2]^-1+probs.curr[,3]^-1+probs.curr[,4]^-1)
       X.6.6<-X.4.4
       
-      V<-1/sum(n)*rbind(cbind(t(X.1.1)%*%X,t(X.1.2)%*%X,t(X.1.3)%*%X,t(X.1.4)%*%X,t(X.1.5)%*%X,t(X.1.6)%*%X),
+      V<-1/n*rbind(cbind(t(X.1.1)%*%X,t(X.1.2)%*%X,t(X.1.3)%*%X,t(X.1.4)%*%X,t(X.1.5)%*%X,t(X.1.6)%*%X),
                         cbind(t(X.1.2)%*%X,t(X.2.2)%*%X,t(X.2.3)%*%X,t(X.2.4)%*%X,t(X.2.5)%*%X,t(X.2.6)%*%X),
                         cbind(t(X.1.3)%*%X,t(X.2.3)%*%X,t(X.3.3)%*%X,t(X.3.4)%*%X,t(X.3.5)%*%X,t(X.3.6)%*%X),
                         cbind(t(X.1.4)%*%X,t(X.2.4)%*%X,t(X.3.4)%*%X,t(X.4.4)%*%X,t(X.4.5)%*%X,t(X.4.6)%*%X),
@@ -415,7 +393,7 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     }
     ##Calculate the GMM loss.
     loss1<-as.vector(t(gbar)%*%invV%*%(gbar))      
-    out1<-list("loss"=max(loss1*sum(n),loss1*sum(n)), "invV"=invV)
+    out1<-list("loss"=loss1, "invV"=invV)
     out1
   }
   gmm.loss<-function(x,...) gmm.func(x,...)$loss
@@ -435,10 +413,10 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     
     w.curr<-cbind(T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] - T4/probs.curr[,4],
                   T1/probs.curr[,1] - T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4],
-                  -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])
+                  -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])/n
     
     ##Generate mean imbalance.
-    loss1<-sum(diag(abs(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))))
+    loss1<-sum(diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr)))
     loss1
   }
   
@@ -451,7 +429,6 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     ##Designate sample size, number of treated and control observations,
     ##theta.curr, which are used to generate probabilities.
     ##Trim probabilities, and generate weights.
-    n<-c(sum(T1), sum(T2), sum(T3), sum(T4))
     theta.curr<-X%*%beta.curr
     baseline.prob<-apply(theta.curr,1,function(x) (1+sum(exp(x)))^-1)
     probs.curr<-cbind(baseline.prob, exp(theta.curr[,1])*baseline.prob, exp(theta.curr[,2])*baseline.prob, exp(theta.curr[,3])*baseline.prob)
@@ -468,39 +445,39 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     
     
     ##Generate the vector of mean imbalance by weights.
-    w.curr.del<-1/sum(n)*t(X)%*%w.curr
+    w.curr.del<-1/n*t(X)%*%w.curr
     w.curr.del<-as.matrix(w.curr.del)
     w.curr<-as.matrix(w.curr)
     
     ##Generate g-bar, as in the paper.
-    gbar<-c(1/sum(n)*t(X)%*%(T2-probs.curr[,2]), 
-            1/sum(n)*t(X)%*%(T3-probs.curr[,3]),
-            1/sum(n)*t(X)%*%(T4-probs.curr[,4]),
+    gbar<-c(1/n*t(X)%*%(T2-probs.curr[,2]), 
+            1/n*t(X)%*%(T3-probs.curr[,3]),
+            1/n*t(X)%*%(T4-probs.curr[,4]),
             w.curr.del)
     
-    dgbar<-rbind(cbind(1/sum(n)*(t(apply(t(X),1,function(x) -x*probs.curr[,2]*(1-probs.curr[,2]))))%*%X,
-                       1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,3])))%*%X,
-                       1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,4])))%*%X,
-                       1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4]))))%*%X,
-                       1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))))%*%X,
-                       1/sum(n)*(t(apply(t(X),1,function(x) x*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))))%*%X
+    dgbar<-rbind(cbind(1/n*t(-X*probs.curr[,2]*(1-probs.curr[,2]))%*%X,
+                       1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
+                       1/n*t(X*probs.curr[,2]*probs.curr[,4])%*%X,
+                       1/n*t(X*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4]))%*%X,
+                       1/n*t(X*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X,
+                       1/n*t(X*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))%*%X
     ),
-    cbind(1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,3])))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) -x*probs.curr[,3]*(1-probs.curr[,3]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,3]*probs.curr[,4])))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))))%*%X
+    cbind(1/n*t(X*probs.curr[,2]*probs.curr[,3])%*%X,
+          1/n*t(-X*probs.curr[,3]*(1-probs.curr[,3]))%*%X,
+          1/n*t(X*probs.curr[,3]*probs.curr[,4])%*%X,
+          1/n*t(X*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4]))%*%X,
+          1/n*t(X*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X,
+          1/n*t(X*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))%*%X
     ),
-    cbind(1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,2]*probs.curr[,4])))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*probs.curr[,3]*probs.curr[,4])))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) -x*probs.curr[,4]*(1-probs.curr[,4]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))))%*%X,
-          1/sum(n)*(t(apply(t(X),1,function(x) x*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))))%*%X
+    cbind(1/n*t(X*probs.curr[,2]*probs.curr[,4])%*%X,
+          1/n*t(X*probs.curr[,3]*probs.curr[,4])%*%X,
+          1/n*t(-X*probs.curr[,4]*(1-probs.curr[,4]))%*%X,
+          1/n*t(X*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
+          1/n*t(X*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X,
+          1/n*t(X*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))%*%X
     ))
     
-    out<-sum(n)*2*dgbar%*%invV%*%gbar
+    out<-2*dgbar%*%invV%*%gbar
     out
   }
   
@@ -519,55 +496,44 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     
     w.curr<-cbind(T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] - T4/probs.curr[,4],
                   T1/probs.curr[,1] - T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4],
-                  -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])
+                  -T1/probs.curr[,1] + T2/probs.curr[,2] - T3/probs.curr[,3] + T4/probs.curr[,4])/n
     
-    dw.beta1<-cbind(t(apply(t(X),1,function(x) x*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4]))))
+    dw.beta1<-cbind(t(X*(T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] - T4*probs.curr[,2]/probs.curr[,4])),
+                    t(X*(T1*probs.curr[,2]/probs.curr[,1] + T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4])),
+                    t(X*(-T1*probs.curr[,2]/probs.curr[,1] - T2*(1 - probs.curr[,2])/probs.curr[,2] - T3*probs.curr[,2]/probs.curr[,3] + T4*probs.curr[,2]/probs.curr[,4])))/n
     
-    dw.beta2<-cbind(t(apply(t(X),1,function(x) x*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4]))))
+    dw.beta2<-cbind(t(X*(T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] - T4*probs.curr[,3]/probs.curr[,4])),
+                    t(X*(T1*probs.curr[,3]/probs.curr[,1] - T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4])),
+                    t(X*(-T1*probs.curr[,3]/probs.curr[,1] + T2*probs.curr[,3]/probs.curr[,2] + T3*(1 - probs.curr[,3])/probs.curr[,3] + T4*probs.curr[,3]/probs.curr[,4])))/n
     
-    dw.beta3<-cbind(t(apply(t(X),1,function(x) x*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))),
-                    t(apply(t(X),1,function(x) x*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4]))))
+    dw.beta3<-cbind(t(X*(T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] + T4*(1 - probs.curr[,4])/probs.curr[,4])),
+                    t(X*(T1*probs.curr[,4]/probs.curr[,1] - T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4])),
+                    t(X*(-T1*probs.curr[,4]/probs.curr[,1] + T2*probs.curr[,4]/probs.curr[,2] - T3*probs.curr[,4]/probs.curr[,3] - T4*(1 - probs.curr[,4])/probs.curr[,4])))/n
     
     loss1<-diag(t(w.curr)%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr))
     
-    out.1<-sapply(2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]), 
-                  function(x) ifelse((x > 0 & loss1[1] > 0) | (x < 0 & loss1[1] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]),
-             function(x) ifelse((x > 0 & loss1[2] > 0) | (x < 0 & loss1[2] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta1[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3]),
-             function(x) ifelse((x > 0 & loss1[3] > 0) | (x < 0 & loss1[3] < 0),abs(x),-abs(x)))
-    out.2<-sapply(2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]), 
-                  function(x) ifelse((x > 0 & loss1[1] > 0) | (x < 0 & loss1[1] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]),
-             function(x) ifelse((x > 0 & loss1[2] > 0) | (x < 0 & loss1[2] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta2[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3]),
-             function(x) ifelse((x > 0 & loss1[3] > 0) | (x < 0 & loss1[3] < 0),abs(x),-abs(x)))
-    out.3<-sapply(2*dw.beta3[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]), 
-                  function(x) ifelse((x > 0 & loss1[1] > 0) | (x < 0 & loss1[1] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta3[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]),
-             function(x) ifelse((x > 0 & loss1[2] > 0) | (x < 0 & loss1[2] < 0),abs(x),-abs(x))) +
-      sapply(2*dw.beta3[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3]),
-             function(x) ifelse((x > 0 & loss1[3] > 0) | (x < 0 & loss1[3] < 0),abs(x),-abs(x)))
+    out.1<-2*dw.beta1[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
+      2*dw.beta1[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
+      2*dw.beta1[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
+    out.2<-2*dw.beta2[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
+      2*dw.beta2[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
+      2*dw.beta2[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
+    out.3<-2*dw.beta3[,1:n]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,1]) +
+      2*dw.beta3[,(n+1):(2*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,2]) +
+      2*dw.beta3[,(2*n+1):(3*n)]%*%X%*%XprimeX.inv%*%t(X)%*%(w.curr[,3])
     out<-c(out.1, out.2, out.3)
     out
   }
   
   n<-length(treat)
-  n.t<-c(sum(T1),sum(T2),sum(T3),sum(T4))
-  x.orig<-x<-cbind(as.matrix(X))
-  
+
   ##Run multionmial logit
   dat.dummy<-data.frame(treat=treat,X)
   #Need to generalize for different dimensioned X's
-  xnam<- paste0("X", 1:dim(X)[2])
+  xnam<- colnames(dat.dummy[,-1])
   fmla <- as.formula(paste("as.factor(treat) ~ -1 + ", paste(xnam, collapse= "+")))
   mnl1<-multinom(fmla, data=dat.dummy,trace=FALSE)
-  mcoef<-matrix(coef(mnl1),k,no.treats-1,byrow=TRUE)
+  mcoef<-t(coef(mnl1))
   mcoef[is.na(mcoef[,1]),1]<-0
   mcoef[is.na(mcoef[,2]),2]<-0
   mcoef[is.na(mcoef[,3]),3]<-0
@@ -583,7 +549,7 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   norms<-apply(probs.mnl,1,sum)
   probs.mnl<-probs.mnl/norms
   mnl1$fit<-matrix(probs.mnl,nrow=n,ncol=no.treats)
-  beta.curr<-mcoef
+  beta.curr<-matrix(mcoef, ncol = 1)
   beta.curr[is.na(beta.curr)]<-0
   
   
@@ -593,10 +559,7 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   ##Generate estimates for balance and CBPSE
   gmm.init<-beta.curr
   
-  if(twostep)
-  {
-    this.invV<-gmm.func(gmm.init)$invV
-  }
+  this.invV<-gmm.func(gmm.init)$invV
   
   if (twostep)
   {
@@ -680,17 +643,13 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     J.opt <- gmm.loss(mcoef)
     warning("Optimization failed.  Results returned are for MLE.")
   }
-  
-  ##Generate weights
-  w.opt<-cbind(T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] - T4/probs.opt[,4],
-               T1/probs.opt[,1] - T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4],
-               -T1/probs.opt[,1] + T2/probs.opt[,2] - T3/probs.opt[,3] + T4/probs.opt[,4])
-  
+
   #How are residuals now defined?
   residuals<-cbind(T1-probs.opt[,1],T2-probs.opt[,2],T3-probs.opt[,3],T4-probs.opt[,4])
   deviance <- -2*c(sum(T1*log(probs.opt[,1])+T2*log(probs.opt[,2])+T3*log(probs.opt[,3])+T4*log(probs.opt[,4])))
   nulldeviance <- -2*c(sum(T1*log(mean(T1))+T2*log(mean(T2))+T3*log(mean(T3))+T4*log(mean(T4))))
   
+  ##Generate weights
   norm1<-norm2<-norm3<-norm4<-1
   if (standardize)
   {
@@ -700,16 +659,11 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
     norm4<-sum(1/probs.opt[which(T4==1),4])
   }
   
-  w<-(T1 == 1)/probs.opt[,1]/norm1 + (T2 == 1)/probs.opt[,2]/norm2 + (T3 == 1)/probs.opt[,3]/norm3 + (T4 == 1)/probs.opt[,4]/norm4
+  w.opt<-(T1 == 1)/probs.opt[,1]/norm1 + (T2 == 1)/probs.opt[,2]/norm2 + (T3 == 1)/probs.opt[,3]/norm3 + (T4 == 1)/probs.opt[,4]/norm4
+  
+
+  W<-gmm.func(beta.opt)$invV
   
-  if (twostep)
-  {
-    W<-this.invV
-  }
-  else
-  {
-    W<-gmm.func(beta.opt)$invV
-  }
   X.G.1.1<-t(-X*probs.opt[,2]*(1-probs.opt[,2]))%*%X
   X.G.1.2<-t(X*probs.opt[,2]*probs.opt[,3])%*%X
   X.G.1.3<-t(X*probs.opt[,2]*probs.opt[,4])%*%X
@@ -745,7 +699,7 @@ CBPS.4Treat<-function(treat, X, X.bal, method, k, XprimeX.inv, bal.only, iterati
   
   class(beta.opt) <- "coef"
   
-  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w, 
+  output<-list("coefficients"=beta.opt,"fitted.values"=probs.opt,"deviance"=deviance,"weights"=w.opt, 
                "y"=treat,"x"=X,"converged"=opt1$conv,"J"=J.opt,"var"=vcov,   
                "mle.J"=ifelse(twostep, gmm.func(mcoef, invV = this.invV)$loss, gmm.loss(mcoef)))
   
